package com.example.gasullaj_proyectofundamentos;

import java.util.ArrayList;

import android.app.Activity;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.gesture.Gesture;
import android.gesture.GestureLibraries;
import android.gesture.GestureLibrary;
import android.gesture.GestureOverlayView;
import android.gesture.GestureOverlayView.OnGesturePerformedListener;
import android.gesture.Prediction;
import android.media.MediaPlayer;
import android.os.Bundle;
import android.support.v4.app.NotificationCompat;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageButton;

public class MainActivity extends Activity implements OnGesturePerformedListener{

	private GestureLibrary libreria;
	
	private Button bJugar;
	private ImageButton bConfigurar, bPuntuaciones, bInfo, bAcercaDe;
	
	private SharedPreferences pref;
	public static AlmacenPuntuaciones almacen;
	private MediaPlayer mp;
	public Posicion posicion;
	private static final int ID_NOTIFICACION_CREAR = 1;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		// Obtenemos las preferencias
		pref = this.getSharedPreferences(
				"com.example.gasullaj_proyectofundamentos_preferences", Context.MODE_PRIVATE);
		
		mp = MediaPlayer.create(this, R.raw.main);
		
		posicion = new Posicion(this);
		
		// Botones de la pantalla principal
		bJugar        = (Button) findViewById(R.id.button_arrancar);
		bConfigurar   = (ImageButton) findViewById(R.id.imagebutton_configurar);
		bAcercaDe     = (ImageButton) findViewById(R.id.imagebutton_acercaDe);
		bInfo         = (ImageButton) findViewById(R.id.imagebutton_info);
		bPuntuaciones = (ImageButton) findViewById(R.id.imagebutton_puntuaciones);
		
		// Listener onClick del botn Jugar
		bJugar.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				lanzarJuego(null);
			}
		});

		// Listener onClick del botn Configurar
		bConfigurar.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				lanzarPreferencias(null);
			}
		});

		// Listener onClick del botn Acerca de
		bAcercaDe.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				lanzarAcercaDe(null);
			}
		});
		
		// Listener onClick del botn Info
	
		bInfo.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				lanzarInfo(null);
			}
		});

		// Listener onClick del botn Puntuaciones
		bPuntuaciones.setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				lanzarPuntuaciones(null);
			}
		});
		
		// Gestures
		libreria = GestureLibraries.fromRawResource(this, R.raw.gestures);
		if (!libreria.load()) {
			finish();
		}
		GestureOverlayView gesturesView = (GestureOverlayView) findViewById(R.id.gestures);
		gesturesView.addOnGesturePerformedListener(this);
		
		// Almacn con las puntuaciones
		almacen = new AlmacenPuntuacionesSQLite(this);
	}
	

	@Override
	protected void onResume() {
		super.onResume();
		// Si hemos activado la msica, y se ha creado correctamente
		if (pref.getBoolean("musica", true) && mp != null) {
			mp.start();
		}

		// Activamos los proveedores de localizacin
		posicion.activarProveedores();
	}

	// Slo si pongo cdigo dentro (msica)
	@Override
	protected void onPause() {		
		// Si hemos activado la msica, y se ha creado correctamente
		if (pref.getBoolean("musica", true) && mp != null) {
			// Para la msica cuando la actividad ya no est activa
			mp.pause();
		}
		
		super.onPause();
		
		// Desactivamos los proveedores de localizacin
		posicion.desactivarProveedores();		
	}
	
	public void lanzarJuego(View view) {
		Intent i = new Intent(this, Juego.class);
		startActivityForResult(i, 1234);
	}
	
	public void lanzarPreferencias(View view) {
		Intent i = new Intent(this, Preferencias.class);
		startActivity(i);
	}
	
	public void lanzarPuntuaciones(View view) {
		Intent i = new Intent(this, Puntuaciones.class);
		startActivity(i);
	}
	
	public void lanzarAcercaDe(View view) {
		Intent i = new Intent(this, AcercaDe.class);
		startActivity(i);
	}
	
	public void lanzarInfo(View view) {
		Intent i = new Intent(this, Instrucciones.class);
		startActivity(i);
	}
	
	public String componerTexto(int puntuacion, 
			String nombre, double longitud, double latitud) {
		String texto = getString(R.string.jugador) + ": " + nombre 
				+ ", " + getString(R.string.puntuacion) + ": " + puntuacion
				+ ", " + getString(R.string.coordenadas) + ": (" + 
				latitud + ", " + longitud + ")";	
		return texto;		
	}
	
	public void mandarCorreo(String direccion, String asunto, String texto) {
		Intent intent = new Intent(Intent.ACTION_SEND);
		intent.setType("text/plain");
		intent.putExtra(Intent.EXTRA_SUBJECT, asunto);
		intent.putExtra(Intent.EXTRA_TEXT, texto);
		intent.putExtra(Intent.EXTRA_EMAIL, new String[] { direccion });
		startActivity(intent);
	}
	
	public void lanzarNotificacion(String title, String text) {
		
		NotificationCompat.Builder notific = new NotificationCompat.Builder(
			this)
			.setContentTitle(title)
			.setSmallIcon(R.drawable.ic_launcher)
			.setContentText(text);
				
		NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notificationManager.notify(ID_NOTIFICACION_CREAR, notific.build());			
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == 1234 && resultCode == RESULT_OK && data != null) {

			int puntuacion = data.getExtras().getInt("puntuacion");
			
			// Obtenemos el nombre desde las preferencias
			String nombre = pref.getString("nombreJugador", getString(R.string.yo));
			
			// Si el nombre est vaco
			if (nombre.trim().length() == 0) {
				// Ponemos el nombre por defecto
				nombre = getString(R.string.yo);
			}
			
			// Obtenemos el correo desde las preferencias
			String correo = pref.getString("correoJugador", "");

			if (correo.trim().length() == 0) {
				correo = "";
			}
					
			double longitud = Posicion.getMejorLocaliz().getLongitude();
			double latitud  = Posicion.getMejorLocaliz().getLatitude();
						
			almacen.guardarPuntuacion(puntuacion, nombre, correo,
					System.currentTimeMillis(), longitud, latitud);
			
			// Enviaremos un correo al jugador y lanzaremos una notificacin,
			// con los datos relativos a la partida que acaba de jugar	
			String asunto = getString(R.string.app_name);
			String texto = componerTexto(puntuacion, nombre, longitud, latitud);
			
			// Si la direccin de correo no est vaca
			if (!(correo.trim().length() == 0)) {
				// Enviamos el correo al jugador 
				mandarCorreo(correo, asunto, texto);			
			}

			// Lanzamos la notificacin
			lanzarNotificacion(asunto, texto);
			
			// Mostramos las puntuaciones
			lanzarPuntuaciones(null);
		}
	}
	
	public void onGesturePerformed(GestureOverlayView ov, Gesture gesture) {
		ArrayList<Prediction> predictions = libreria.recognize(gesture);
		if (predictions.size() > 0) {
			String comando = predictions.get(0).name;
			if (comando.equals("play")) {
				lanzarJuego(null);
			} else if (comando.equals("configurar")) {
				lanzarPreferencias(null);
			} else if (comando.equals("acerca_de")) {
				lanzarAcercaDe(null);
			} else if (comando.equals("info")) {
				lanzarInfo(null);
			} else if (comando.equals("puntos")) {
				lanzarPuntuaciones(null);
			} 
		}
	}
		
	@Override
	protected void onSaveInstanceState(Bundle estadoGuardado) {
		super.onSaveInstanceState(estadoGuardado);
		// Aado la comprobacin de que la msica est activada
		if (mp != null && pref.getBoolean("musica", true)) { 
			int pos = mp.getCurrentPosition(); 
			estadoGuardado.putInt("posicion", pos); 
		}
	}

	@Override
	protected void onRestoreInstanceState(Bundle estadoGuardado) {
		super.onRestoreInstanceState(estadoGuardado);
		// Aado la comprobacin de que la msica est activada
		if (estadoGuardado != null && mp != null 
				&& pref.getBoolean("musica", true)) { 
			int pos = estadoGuardado.getInt("posicion"); mp.seekTo(pos);
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		switch (item.getItemId()) {

		case R.id.itemInfo:
            lanzarInfo(null);
            break;			
        
		case R.id.itemConfig:
            lanzarPreferencias(null);
            break;
            
		case R.id.itemPuntos:
            lanzarPuntuaciones(null);
            break;           
		}
		return super.onOptionsItemSelected(item);
	}
}
