package com.example.gasullaj_proyectofundamentos;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;

public class Posicion implements LocationListener{

	private static final long TIEMPO_REDES = 10 * 1000; // 10 segundos
	private static final long TIEMPO_GPS   = 20 * 1000; // 20 segundos
	private static final long DISTANCIA_GPS   = 5;  //  5 metros
	private static final long DISTANCIA_REDES = 10; // 10 metros
	private static final long DOS_MINUTOS = 2 * 60 * 1000;
	
	public LocationManager manejador;
	private static Location mejorLocaliz;
	
	private Context context;
	
	public Posicion(Context context) {			
		this.context = context;
		
		manejador = (LocationManager) context.getSystemService(context.LOCATION_SERVICE);
		
		if (!manejador.isProviderEnabled(LocationManager.GPS_PROVIDER) && 
			!manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			mostrarAlertaConfiguracion();
		}
		
		if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			actualizaMejorLocaliz(manejador
					.getLastKnownLocation(LocationManager.GPS_PROVIDER));
		}
		if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			actualizaMejorLocaliz(manejador
					.getLastKnownLocation(LocationManager.NETWORK_PROVIDER));			
		}
	}
	
	private void actualizaMejorLocaliz(Location localiz) {
		if (localiz != null && (mejorLocaliz == null || 
				     localiz.getAccuracy() < 2 * mejorLocaliz.getAccuracy() || 
				     localiz.getTime() - mejorLocaliz.getTime() > DOS_MINUTOS)) {
			mejorLocaliz = localiz;
		}
	}
	
	public void activarProveedores() {
		if (manejador.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
			manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER,
					TIEMPO_GPS, DISTANCIA_GPS, this);
		}
		if (manejador.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
			manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
					TIEMPO_REDES, DISTANCIA_REDES, this);
		}
	}
	
	public void desactivarProveedores() {
		manejador.removeUpdates(this);
	}
	
	public void mostrarAlertaConfiguracion(){
		new AlertDialog.Builder(context)	
		.setTitle(context.getResources().getString(R.string.alertaLocationConfigTitle))
		.setMessage(context.getResources().getString(R.string.alertaLocationConfigText))
		.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
		public void onClick(DialogInterface dialog, int boton){
		Intent i = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
		context.startActivity(i);
		}})
		.setNegativeButton(android.R.string.cancel, null)
		.show();
	}
		
	public static Location getMejorLocaliz() {
		return mejorLocaliz;
	}

	public static void setMejorLocaliz(Location mejorLocaliz) {
		Posicion.mejorLocaliz = mejorLocaliz;
	}

	// Mtodos de la interfaz LocationListener
	public void onLocationChanged(Location location) {
		actualizaMejorLocaliz(location);					
	}	
	public void onProviderDisabled(String proveedor) {
		activarProveedores();
	}

	public void onProviderEnabled(String proveedor) {
		activarProveedores();
	}

	public void onStatusChanged(String proveedor, int estado, Bundle extras) {
		activarProveedores();
	}	
}
