package com.example.gasullaj_proyectofundamentos;

import java.util.List;
import java.util.Vector;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Path.Direction;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.ShapeDrawable;
import android.graphics.drawable.shapes.PathShape;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.os.Vibrator;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;


public class VistaJuego extends View implements SensorEventListener {
	
	// //// THREAD Y TIEMPO //////
	// Thread encargado de procesar el juego
	private ThreadJuego thread = new ThreadJuego();
	// Cada cuanto queremos procesar cambios (ms)
	private static int PERIODO_PROCESO = 50;
	// Cuando se realiz el ltimo proceso
	private long ultimoProceso = 0;
	
	// //// GRFICOS ///////
	private static final int VELOCIDAD_AVANCE = 17;
	private static final int QUIETO = 0;    // El protagonista no se mueve
	private static final int AVANCE = 1;    // El protagonista avanza
	private static final int RETROCESO = 2; // El protagonista retrocede
	private int anchoPantalla = 0;	// Guardaremos el ancho de la pantalla
	private int altoPantalla  = 0;	// Guardaremos el alto  de la pantalla
    private int movimiento = QUIETO; // Indicaremos el tipo de movimiento. Inicialmente quieto
    private int refPosXElementos; // Referencia de los elementos, para retroceso del prota
    
    private Drawable drawableRetroceso, drawableAvance, drawableSalto;
    private Grafico avance, retroceso, salto;
    
    // //// PROTAGONISTA //////
    private Grafico prota;  // Grfico del protagonista
	private double velXProta = 0; // Velocidad del protagonista
	private double velYProta = 0;
	private static final int VELOCIDAD_SALTO = 15;
	boolean saltoPendiente = false;
	boolean colisionProtaObstaculo = false;
	private int vidas = 3;
	private int puntuacion = 0;

	// //// PLATAFORMAS //////
    private Vector<Grafico> plataformas;	// Vector con los plataformas
    private static final int numPlataformas = 15;		// Nmero de plataformas
	private static final int numTiposPlataformas = 4;	// Tipos de plataformas 
	private Drawable drawablePlataforma[]= new Drawable[numTiposPlataformas];
	
	// //// TESOROS //////
    private Vector<Grafico> tesoros;	// Vector con los tesoros
    // Nmero de tesoros:
    // La 1 mitad se situar sobre plataformas
    // La 2 mitad se situar sobre el suelo
    private static final int numTesoros = 2 * numPlataformas; 
    private static final int numTiposTesoros = 2;	// Tipos de tesoros
	private Drawable drawableTesoro[]= new Drawable[numTiposTesoros];	
	
	// //// OBSTCULOS //////
    private Vector<Grafico> obstaculos;	// Vector con los obstaculos
    private static final int numObstaculos = numPlataformas; 
    private Drawable drawableObstaculo;
    
    // //// ESCUDOS //////
    private Vector<Grafico> escudos;	// Vector con los escudos
    private static final int numEscudos = 2;
    private Drawable drawableEscudo;
    
	double velXElementos = 0; // Velocidad de los elementos
	double velYElementos = 0; // diferentes al prota	

	// //// SENSORES //////
	private SensorManager mSensorManager;
	private Sensor sensor;
	private static final float GRAVEDAD = (float) 9.81;
	// Para manejo con sensor de orientacin
	private boolean hayValorInicialY = false; 
	private float valorInicialY; 
	// Flag que indica que se ha activado el sensor de proximidad
	private boolean flagProx = false;  
  
	// //// PREFERENCIAS //////
	private SharedPreferences pref;
	private int manejo; // Indica como manejar al prota
	
	// //// MULTIMEDIA //////
	SoundPool soundPool;
	int idEscudo, idMuerte, idSalto, idTesoro0, idTesoro1, idObstaculo;
	
	private Activity padre;
	private Context context;
	
	public VistaJuego(Context context, AttributeSet attrs) {
		super(context, attrs);
		
		this.context = context;
				
		Drawable drawableProta;

		// Asignamos el foco a la vista para poder recoger eventos de teclado
		setFocusable(true);
		setFocusableInTouchMode(true);
		requestFocus();	
		
		// Obtenemos las preferencias
		pref = context.getSharedPreferences(
				"com.example.gasullaj_proyectofundamentos_preferences", Context.MODE_PRIVATE);
		
		// Obtenemos el valor que nos indicar como manejar al prota
		manejo = Integer.valueOf(pref.getString("manejo", "0"));
		
		// Dibujo de avance
		Path pathAvance = new Path();
		pathAvance.moveTo((float) 0.0, (float) 0.0);
		pathAvance.lineTo((float) 0.5, (float) 0.5);
		pathAvance.lineTo((float) 0.0, (float) 1.0);
		pathAvance.lineTo((float) 0.0, (float) 0.0);
		ShapeDrawable dAvance = new ShapeDrawable(new PathShape(pathAvance, 1,
				1));
		dAvance.getPaint().setColor(Color.WHITE);
		dAvance.getPaint().setStyle(Paint.Style.FILL);
		dAvance.setIntrinsicWidth(100);
		dAvance.setIntrinsicHeight(100);
		drawableAvance = dAvance;

		// Dibujo de retroceso
		Path pathRetroceso = new Path();
		pathRetroceso.moveTo((float) 0.5, (float) 0.0);
		pathRetroceso.lineTo((float) 0.0, (float) 0.5);
		pathRetroceso.lineTo((float) 0.5, (float) 1.0);
		pathRetroceso.lineTo((float) 0.5, (float) 0.0);
		ShapeDrawable dRetroceso = new ShapeDrawable(new PathShape(
				pathRetroceso, 1, 1));
		dRetroceso.getPaint().setColor(Color.WHITE);
		dRetroceso.getPaint().setStyle(Paint.Style.FILL);
		dRetroceso.setIntrinsicWidth(100);
		dRetroceso.setIntrinsicHeight(100);
		drawableRetroceso = dRetroceso;

		// Dibujo de salto
		Path pathSalto = new Path();
		pathSalto.moveTo((float) 0.5, (float) 0.0);
		pathSalto.lineTo((float) 1.0, (float) 0.5);
		pathSalto.lineTo((float) 0.0, (float) 0.5);
		pathSalto.lineTo((float) 0.5, (float) 0.0);
		ShapeDrawable dSalto = new ShapeDrawable(new PathShape(pathSalto, 1, 1));
		dSalto.getPaint().setColor(Color.WHITE);
		dSalto.getPaint().setStyle(Paint.Style.FILL);
		dSalto.setIntrinsicWidth(100);
		dSalto.setIntrinsicHeight(100);
		drawableSalto = dSalto;	 
		
		avance    = new Grafico(this, drawableAvance);
		retroceso = new Grafico(this, drawableRetroceso);
		salto     = new Grafico(this, drawableSalto);
		
		// Parte relacionada con el protagonista		
		// Graficos vectoriales
		if (pref.getString("graficos", "1").equals("0")) {
			Path pathProta = new Path();
			pathProta.addRect (0.0f, 0.0f, 1.0f, 1.0f, Direction.CW);
			ShapeDrawable dProta = new ShapeDrawable(new PathShape(
					pathProta, 1, 1));
			dProta.getPaint().setColor(Color.RED);
			dProta.getPaint().setStyle(Paint.Style.FILL);
			dProta.setIntrinsicWidth(50);
			dProta.setIntrinsicHeight(50);

			drawableProta = dProta;
			
			setBackgroundColor(Color.BLACK);
		// Bitmap
		} else {
			drawableProta = context.getResources().getDrawable(R.drawable.prota);
			setBackgroundDrawable(context.getResources().getDrawable(R.drawable.fondo));
		}
				
		prota = new Grafico(this, drawableProta);		
		
		// Parte relacionada con las plataformas
		if (pref.getString("graficos", "1").equals("0")) {
			Path pathPlataforma = new Path();
			pathPlataforma.addRect (0.0f, 0.0f, 1.0f, 0.4f, Direction.CW);

			for (int i = 0; i < numTiposPlataformas; i++) {
				ShapeDrawable dPlataforma = new ShapeDrawable(new PathShape(
						pathPlataforma, 1, 1));
				dPlataforma.getPaint().setColor(Color.GREEN);
				dPlataforma.getPaint().setStyle(Paint.Style.FILL);

				// Fijamos el ancho de cada tipo de plataforma 
				// Las de menor ndice son mayores
				dPlataforma.setIntrinsicWidth(((numTiposPlataformas- i) * 70));
				dPlataforma.setIntrinsicHeight(25);
				drawablePlataforma[i] = dPlataforma;
			}
		} else {
			for (int i = 0; i < numTiposPlataformas; i++) {
				// Solo hay dos tipos de drawables para plataforma (son del mismo tamao)				
				if (i <= 1) {
					drawablePlataforma[i] = context.getResources().getDrawable(R.drawable.plataforma0);
				} else {
					drawablePlataforma[i] = context.getResources().getDrawable(R.drawable.plataforma1);
				}
			}
		}

		plataformas = new Vector<Grafico>();
		for (int i = 0; i < numPlataformas; i++) {
			// El tipo de las plataformas se fija de manera aleatoria
			int tipoPlataforma = (int) Math.round(Math.random()*(numTiposPlataformas-1));
			Grafico plataforma = new Plataforma(this, drawablePlataforma[tipoPlataforma], tipoPlataforma);			
			plataformas.add(plataforma);
		}
				
		// Parte relacionada con las tesoros
		if (pref.getString("graficos", "1").equals("0")) {
			Path pathTesoro = new Path();
			pathTesoro.addRect (0.0f, 0.0f, 1.0f, 1.0f, Direction.CW);
			for (int i = 0; i < numTiposTesoros; i++) {
				ShapeDrawable dTesoro = new ShapeDrawable(new PathShape(
						pathTesoro, 1, 1));
				if (i == 0) {
					// Tesoro tipo 0				
					dTesoro.getPaint().setColor(Color.YELLOW);
				} else {
					// Tesoro tipo 1
					dTesoro.getPaint().setColor(Color.MAGENTA);			
				}
				dTesoro.getPaint().setStyle(Paint.Style.FILL);
				dTesoro.setIntrinsicWidth(25);
				dTesoro.setIntrinsicHeight(25);
				drawableTesoro[i] = dTesoro;
			}

		} else {
			for (int i = 0; i < numTiposTesoros; i++) {
				// Asignamos el tesoro con ms puntos al tipo 0
				if (i == 0) {
					drawableTesoro[i] = context.getResources().getDrawable(R.drawable.tesoro0);
				} else{
					drawableTesoro[i] = context.getResources().getDrawable(R.drawable.tesoro1);
				}
			}
		}

		tesoros = new Vector<Grafico>();
		for (int i = 0; i < numTesoros; i++) {
			// El tipo de las tesoros se fija de manera aleatoria
			int tipoTesoro = (int) Math.round(Math.random()*(numTiposTesoros-1));
			Grafico tesoro = new Tesoro(this, drawableTesoro[tipoTesoro], tipoTesoro);			
			tesoros.add(tesoro);
		}
			
		// Parte relacionada con las obstaculos
		if (pref.getString("graficos", "1").equals("0")) {
			Path pathObstaculo = new Path();
		    float ref= 0.7f;
			pathObstaculo.moveTo(ref * 0.5f, ref * 0.84f); // Punta arriba izqda
			pathObstaculo.lineTo(ref * 1.5f, ref * 0.84f); // Punta arriba dcha
			pathObstaculo.lineTo(ref * 0.68f, ref * 1.45f); // Punta abajo izqda
			pathObstaculo.lineTo(ref * 1.0f, ref * 0.5f); // Punta centro arriba
			pathObstaculo.lineTo(ref * 1.32f, ref * 1.45f); // Punta abajo dcha
			pathObstaculo.lineTo(ref * 0.5f, ref * 0.84f); // Punta arriba izqda

			ShapeDrawable dObstaculo = new ShapeDrawable(new PathShape(
					pathObstaculo, 1, 1));
			dObstaculo.getPaint().setColor(Color.LTGRAY);
			dObstaculo.getPaint().setStyle(Paint.Style.FILL);
			dObstaculo.setIntrinsicWidth(50);
			dObstaculo.setIntrinsicHeight(50);
			drawableObstaculo = dObstaculo;
		} else {
			drawableObstaculo = context.getResources().getDrawable(R.drawable.obstaculo);
		}

		obstaculos = new Vector<Grafico>();
		for (int i = 0; i < numObstaculos; i++) {
			Grafico obstaculo = new Grafico(this, drawableObstaculo);			
			obstaculos.add(obstaculo);
		}			

		// Parte relacionada con las escudos
		if (pref.getString("graficos", "1").equals("0")) {
			Path pathEscudo = new Path();

			pathEscudo.moveTo(0.0f, 0.0f);
			pathEscudo.lineTo(1.0f, 0.0f);
			pathEscudo.lineTo(0.5f, 0.5f);
			pathEscudo.lineTo(0.0f, 0.0f);

			ShapeDrawable dEscudo = new ShapeDrawable(new PathShape(
					pathEscudo, 1, 1));
			dEscudo.getPaint().setColor(Color.GREEN);
			dEscudo.getPaint().setStyle(Paint.Style.FILL);
			dEscudo.setIntrinsicWidth(50);
			dEscudo.setIntrinsicHeight(50);
			drawableEscudo = dEscudo;
		} else {
			drawableEscudo = context.getResources().getDrawable(R.drawable.escudo);
		}

		escudos = new Vector<Grafico>();
		for (int i = 0; i < numEscudos; i++) {
			Grafico escudo = new Grafico(this, drawableEscudo);			
			escudos.add(escudo);
		}	
		
		// Parte relacionada con los sensores		
		mSensorManager = (SensorManager) context.getSystemService(Context.SENSOR_SERVICE);	
		
		// Parte relacionada con SoundPool. Activamos los sonidos,
		// si hemos habilitado la msica por preferencias  
		if (pref.getBoolean("musica", true)) {
			soundPool   = new SoundPool(5, AudioManager.STREAM_MUSIC, 0);
			idEscudo    = soundPool.load(context, R.raw.escudo, 0);
			idMuerte    = soundPool.load(context, R.raw.muerte, 0);
			idSalto     = soundPool.load(context, R.raw.salto, 0);
			idTesoro0   = soundPool.load(context, R.raw.tesoro0, 0);
			idTesoro1   = soundPool.load(context, R.raw.tesoro1, 0);
			idObstaculo = soundPool.load(context, R.raw.obstaculo, 0);
		}			
	}

	@Override
	protected void onSizeChanged(int ancho, int alto, int ancho_anter,
			int alto_anter) {
		super.onSizeChanged(ancho, alto, ancho_anter, alto_anter);
		// Una vez que conocemos nuestro ancho y alto.
		
		// Guardamos el tamao de la pantalla
		anchoPantalla = ancho;
		altoPantalla = alto;
			
		// Si hemos habilitado el uso de la pantalla tctil
		if (manejo == 0) {
			// Posicionamos sobre la pantalla
			// los controles de avance, retroceso y salto
			retroceso.setCenX(retroceso.getAncho());
			retroceso.setCenY(retroceso.getAlto() / 2 + salto.getAlto());
			avance.setCenX(retroceso.getCenX() + retroceso.getAncho() / 2
					+ salto.getAncho() + avance.getAncho() / 2);
			avance.setCenY(retroceso.getCenY());
			salto.setCenX(retroceso.getCenX() + retroceso.getAncho() / 2
					+ salto.getAncho() / 4);
			salto.setCenY(salto.getAlto());
		}
				
		// Posicionamos al protagonista		
		prota.setCenX(prota.getAncho()/2);
		prota.setCenY(alto - prota.getAlto()/2);
		
		// Posicionamos las plataformas, los tesoros y los obstculos 
		for (int i = 0; i < plataformas.size(); i++) {
			Grafico plataforma = plataformas.elementAt(i);
			plataforma.setCenX((int) ((ancho * 0.65 * (1 + 0.8*i)) + plataforma.getAncho()/2));
			// Evitamos que la plataforma se situe en la 
			// parte superior o inferior de la pantalla
			plataforma.setCenY((int) (alto * (0.25 + Math.random() * 0.5)));
			
			// Contendr las coordenadas en las que colocar el tesoro
			int centroXTesoro;
			int centroYTesoro;
			Grafico tesoro;
			
			// Colocamos un tesoro sobre la plataforma
			// (los tesoros con ndice 0 a plataformas.size()-1)
			tesoro = tesoros.elementAt(i);			
			centroXTesoro = plataforma.getCenX() - plataforma.getAncho() / 2 
					+ tesoro.getAncho() / 2
					+ (int) Math.round(Math.random() * (plataforma.getAncho() - tesoro.getAncho()));		
			centroYTesoro = plataforma.getCenY() - plataforma.getAlto() / 2
					- tesoro.getAlto() / 2;
			tesoro.setCenX(centroXTesoro);
			tesoro.setCenY(centroYTesoro);
			
			// Colocamos un tesoro sobre el suelo (cerca de la plataforma)
			// (los tesoros de plataformas.size() hasta el final)
			tesoro = tesoros.elementAt(plataformas.size() + i);
			centroXTesoro = plataforma.getCenX() 
					+ (int) Math.round(Math.random() * anchoPantalla/2);
			centroYTesoro = altoPantalla - tesoro.getAlto() / 2;
			tesoro.setCenX(centroXTesoro);
			tesoro.setCenY(centroYTesoro);
			
			// Contendr las coordenadas en las que colocar el obstaculo
			int centroXObstaculo;
			int centroYObstaculo;
			Grafico obstaculo;
			
			// Colocamos un obstaculo sobre la plataforma
			// (los obstaculos con ndice 0 a plataformas.size()-1)
			obstaculo = obstaculos.elementAt(i);	
			
			// Si el tesoro est situado en la segunda mitad de la plataforma
			// y la plataforma no es muy pequea
			if ( (tesoros.elementAt(i).getCenX() - plataforma.getCenX() > 0) && 
				(((Plataforma) plataforma).getTipoPlataforma() <= 1) ) {
				// Situamos el obstculo en la primera mitad							
				centroXObstaculo = plataforma.getCenX() - plataforma.getAncho() / 2 
						+ obstaculo.getAncho() / 2
						+ (int) Math.round(Math.random() * (plataforma.getAncho() / 2 - obstaculo.getAncho()));
				centroYObstaculo = plataforma.getCenY() - plataforma.getAlto() / 2
						- obstaculo.getAlto() / 2;								
			// Si el tesoro est en la primera mitad y la plataforma no es muy pequea
			} else if (((Plataforma) plataforma).getTipoPlataforma() <= 1) {
				// Situamos el obstculo en la segunda mitad				
				centroXObstaculo = plataforma.getCenX() + obstaculo.getAncho() / 2						
						+ (int) Math.round(Math.random() * (plataforma.getAncho() / 2 - obstaculo.getAncho()));
				centroYObstaculo = plataforma.getCenY() - plataforma.getAlto() / 2
						- obstaculo.getAlto() / 2;									
			// Colocamos el obstculo sobre el suelo
			} else {
				// Separo el obstculo del ltimo tesoro que he colocado en el suelo
				centroXObstaculo = centroXTesoro - tesoro.getAncho()/2  - obstaculo.getAncho()/2 
						- (int) Math.round(Math.random() * anchoPantalla/4) - 5; 
				centroYObstaculo = altoPantalla - obstaculo.getAlto() / 2;				
			}
			
			obstaculo.setCenX(centroXObstaculo);
			obstaculo.setCenY(centroYObstaculo);						
		}
						
		// Posicionamos los escudos
		posicionaEscudo(0);
		posicionaEscudo(1);
		
		// Obtenemos una referencia a la posicin inicial de los 
		// elementos, a utilizar cuando el protagonista retroceda
		refPosXElementos = plataformas.elementAt(0).getCenX();
		
		ultimoProceso = System.currentTimeMillis();
		thread.start();
	}

	@Override
	protected synchronized void onDraw(Canvas canvas) {
		super.onDraw(canvas);
					
		// Dibujamos los tesoros
		for (Grafico tesoro : tesoros) {
			tesoro.dibujaGrafico(canvas);
		}
		
		// Dibujamos los obstaculos
		for (Grafico obstaculo : obstaculos) {
			obstaculo.dibujaGrafico(canvas);			
		}
		
		// Dibujamos los escudos
		for (Grafico escudo : escudos) {
			escudo.dibujaGrafico(canvas);
		}
		
		// Dibujamos al protagonista
		prota.dibujaGrafico(canvas);
		
		// Dibujamos las plataformas
		for (Grafico plataforma : plataformas) {
			plataforma.dibujaGrafico(canvas);
		}
		
		// Dibujamos los controles de avance, retroceso y salto
		retroceso.dibujaGrafico(canvas);
		avance.dibujaGrafico(canvas);
		salto.dibujaGrafico(canvas);
	}
	
	protected synchronized void actualizaFisica() {
		long ahora = System.currentTimeMillis();
		
		if (ultimoProceso + PERIODO_PROCESO > ahora) {
			return; // Salir si el perodo de proceso no se ha cumplido.
		}
		
		// Para una ejecucin en tiempo real calculamos retardo
		double retardo = (ahora - ultimoProceso) / PERIODO_PROCESO;
		ultimoProceso = ahora; // Para la prxima vez
		
		switch (movimiento) {	
		case QUIETO:
			velXProta = 0;
			velXElementos = 0;
			break;
		case AVANCE:
			// Si el protagonista no ha llegado a la mitad de la pantalla
			if (prota.getCenX() < anchoPantalla/2) {
				// El protagonista avanza
				velXProta = VELOCIDAD_AVANCE;
				// El resto de elementos no se mueve
				velXElementos = 0;
			// Si ya hemos recorrido todas las plataformas
			} else if (plataformas.elementAt(plataformas.size()-1).getCenX() 
						< (anchoPantalla * 0.25)) {
				// Los elementos no se mueven
				velXElementos = 0;
				// Si el protagonista no ha llegado al final de la pantalla
				if ( (prota.getCenX() + prota.getAncho()/2) < anchoPantalla * 0.99){
					// El protagonista avanza
					velXProta = VELOCIDAD_AVANCE;
				} else {
					// El protagonista se detiene
					velXProta = 0;
				}
			// Vamos recorriendo plataformas	
			} else {
				// El protagonista se detiene
				velXProta = 0;
				// El resto de elementos se mueven
				velXElementos = -VELOCIDAD_AVANCE;
			}
			break;
		case RETROCESO:
			// Si los elementos no estn en su posicin inicial
			if (plataformas.elementAt(0).getCenX() < refPosXElementos) {
				// Si el protagonista est al final
				if (prota.getCenX() > anchoPantalla/2) {
					// El protagonista retrocede
					velXProta = -VELOCIDAD_AVANCE;
					// El resto de elementos no se mueve
					velXElementos = 0;	
				} else {
					// El protagonista se detiene
					velXProta = 0;
					// El resto de elementos avanzan
					velXElementos = VELOCIDAD_AVANCE;					
				}		
			} else {
				// El resto de elementos no se mueve
				velXElementos = 0;
				
				// Si el protagonista no ha llegado al principio de la pantalla
				if (prota.getCenX() > prota.getAncho()/2) {
					// El protagonista retrocede
					velXProta = -VELOCIDAD_AVANCE;
				} else {
					// El protagonista se detiene
					velXProta = 0;
				}								
			}
			break;
		}					
		
		// Impedimos que el prota tras saltar salga
		// de la pantalla por la parte superior
		if ( prota.getCenY() <= (prota.getAlto()/2) ) {
			prota.setCenY(prota.getAlto()/2);
			if (prota.getIncY() < 0) {
				// Para que el prota empiece a descender
				velYProta = 0;
			}
		}
		
		// Impedimos que el prota tras caer traspase el suelo
		if ( prota.getCenY() > (altoPantalla - (prota.getAlto()/2)) ) {
			prota.setCenY(altoPantalla - (prota.getAlto()/2));
		}
		
		// Verificamos si hay colisin de prota y plataforma y 
		// variamos velocidad y posicin del prota, si es necesario
		for (Grafico plataforma : plataformas) {
			// Si la plataforma est en la pantalla
			if (plataforma.visible(anchoPantalla)) {
				// Verificamos si ha colisionado con el prota
				switch (((Plataforma) plataforma).verificaColisionLados(prota)) {
					case Plataforma.COLISION_NO:
						// Indicamos que el prota no est sobre la plataforma
						((Plataforma) plataforma).setProtaEnPlat(false);
						break;
						
					case Plataforma.COLISION_ARRIBA:
						// Si no hay un salto pendiente o no acaba de saltar
						if ( (!saltoPendiente) || (velYProta >= 0) ) {
							// Para que el Prota se mantenga
							// sobre la plataforma
							velYProta = 0;
							
							// Situamos al prota justo un pixel dentro de la plataforma
							prota.setCenY(plataforma.getCenY() - 
									plataforma.getAlto()/2 - prota.getAlto()/2 + 1);
						}						
						
						// Indicamos que el prota est sobre la plataforma
						((Plataforma) plataforma).setProtaEnPlat(true);
						break;
						
					case Plataforma.COLISION_ABAJO:
						// Si el prota est ascendiendo
						if (velYProta < 0) {
							// Hacemos que empiece a descender
							velYProta = velYProta * (-1);	
						}
						// Indicamos que el prota no est sobre la plataforma
						((Plataforma) plataforma).setProtaEnPlat(false);
						break;
						
					case Plataforma.COLISION_IZQUIERDA:
						velXProta = 0;
						// Indicamos que el prota no est sobre la plataforma
						((Plataforma) plataforma).setProtaEnPlat(false);
						break;
						
					case Plataforma.COLISION_DERECHA:
						velXProta = 0;
						// Indicamos que el prota no est sobre la plataforma
						((Plataforma) plataforma).setProtaEnPlat(false);
						break;	
				}		
			}
		}		
		
		// Si el prota no est pisando suelo ni plataforma
		if (!protaSueloOPlat()) {
			// La velocidad Y va disminuyendo
			velYProta += 0.0151 * VELOCIDAD_SALTO;
		// Sino si no hay un salto pendiente o el
		// prota est descendiendo
		} else if ( (!saltoPendiente) || (velYProta > 0) ) {
			velYProta = 0;
		}
								
		// Actualizamos velocidad y posicin de las plataformas		
		for (Grafico plataforma : plataformas) {	
			plataforma.setIncX(velXElementos);
			plataforma.setIncY(velYElementos);
			plataforma.incrementaPos(retardo);
		}
		
		// Actualizamos velocidad y posicin de los tesoros		
		for (Grafico tesoro : tesoros) {	
			tesoro.setIncX(velXElementos);
			tesoro.setIncY(velYElementos);
			tesoro.incrementaPos(retardo);
		}		
		
		// Actualizamos velocidad y posicin de los obstaculos		
		for (Grafico obstaculo : obstaculos) {	
			obstaculo.setIncX(velXElementos);
			obstaculo.setIncY(velYElementos);
			obstaculo.incrementaPos(retardo);
		}
		
		// Actualizamos velocidad y posicin de los escudos		
		for (Grafico escudo : escudos) {	
			escudo.setIncX(velXElementos);
			escudo.setIncY(velYElementos);
			escudo.incrementaPos(retardo);
		}
		
		// Actualizamos velocidad y posicin del protagonista
		prota.setIncX(velXProta);
		prota.setIncY(velYProta);						
		prota.incrementaPos(retardo);
		
		// Verificamos si hay colisin de prota y algn tesoro
		verificaColisionProtaTesoros();

		// Verificamos si hay colisin de prota y algn obstculo
		verificaColisionProtaObstaculos();
		
		// Verificamos si hay colisin de prota y algn escudo
		verificaColisionProtaEscudos();
				
		// Indicamos que no hay ningn
		// salto pendiente
		saltoPendiente = false;
	}

	public void protaSalta() {
		// Si el prota est tocando suelo
		// o sobre una plataforma
		if (protaSueloOPlat())  {
			// El prota salta
			velYProta = -VELOCIDAD_SALTO;
			
			// Activamos los sonidos, si hemos
			// habilitado la msica por preferencias
			if (pref.getBoolean("musica", true)) { 
				// Reproducimos el sonido asociado 
				soundPool.play(idSalto, 1, 1, 0, 0, 1); 
			}
			
			// Indicamos que el prota va a saltar
			saltoPendiente = true;			
		}
	}

	public boolean protaSueloOPlat() {
		// Inicialmente decimos que el prota
		// no pisa tierra
		boolean protaSueloPlat = false;
		
		// Si el prota est en el suelo 
		if (prota.getCenY() >= (altoPantalla - prota.getAlto()/2)) {
			// Indicamos que est pisando tierra
			protaSueloPlat = true;	
		}
		
		// Recorremos las plataformas
		for (Grafico plataforma : plataformas) {
			// Si la plataforma est en la pantalla
			if (plataforma.visible(anchoPantalla)) {
				// Si el prota est sobre una plataforma
				if (((Plataforma) plataforma).isProtaEnPlat()) {
					// Indicamos que est pisando tierra
					protaSueloPlat = true;
				}
			}
		}

		return protaSueloPlat;
	}	
	
	// Verificamos si hay colisin de prota y algn tesoro
	private void verificaColisionProtaTesoros() {
		if (tesoros.size() > 0) {
			// Recorremos tesoros en orden inverso
			for (int i = tesoros.size() - 1; i >= 0; i--) {
				Grafico tesoro = tesoros.elementAt(i);
				// Si el tesoro est en la pantalla
				if (tesoro.visible(anchoPantalla)) {
					// Si ha colisionado con el prota
					if (prota.verificaColision(tesoro)) {
						// Destruimos el tesoro
						destruyeTesoro(i);
					}
				}
			}
		}
	}	
	
	private void destruyeTesoro(int i) {				
		// Incrementamos la puntuacin en funcin
		// del tipo de tesoro
		switch (((Tesoro) tesoros.elementAt(i)).getTipoTesoro()) {
			case 0:
				puntuacion += 1000;		

				// Activamos los sonidos, si hemos
				// habilitado la msica por preferencias
				if (pref.getBoolean("musica", true)) {
					// Reproducimos el sonido asociado
					soundPool.play(idTesoro0, 1, 1, 0, 0, 1);
				}
		
				break;

			default:
				puntuacion += 100;
				
				// Activamos los sonidos, si hemos
				// habilitado la msica por preferencias
				if (pref.getBoolean("musica", true)) {
					// Reproducimos el sonido asociado
					soundPool.play(idTesoro1, 1, 1, 0, 0, 1);
				}
				
				break;
		}
				
		// Eliminamos el tesoro
		tesoros.remove(i);
				
		// Si ya hemos cogido todos los tesoros
		if (tesoros.isEmpty()) {
			// Salimos del juego
			salir();
		}
	}
		
	// Verificamos si hay colisin de prota y algn obstculo
	public void verificaColisionProtaObstaculos() {
		// En principio indicamos que no hay colisin
		boolean colision = false;
		for (Grafico obstaculo : obstaculos) {
			// Si el obstaculo est en la pantalla
			if (obstaculo.visible(anchoPantalla)) {
				// Si ha colisionado con el prota
				if (prota.verificaColision(obstaculo)) {
					// Ha habido colisin
					colision = true;
					
					// Activamos los sonidos, si hemos
					// habilitado la msica por preferencias
					if (pref.getBoolean("musica", true)) { 
						// Reproducimos el sonido asociado 
						soundPool.play(idObstaculo, 1, 1, 0, 0, 1); 
					}

					// El terminal vibra
					 Vibrator v = (Vibrator) this.context.getSystemService(Context.VIBRATOR_SERVICE);
					// Se establece el periodo de vibracin 
					v.vibrate(PERIODO_PROCESO);
									
					// Si es la 1 vez que detectamos esta colisin
					if (colisionProtaObstaculo == false) {
						// Marcamos la colisin para no volver a restar 
						// vidas al seguir detectando la misma colisin
						colisionProtaObstaculo = true;
						// Quitamos una vida
						vidas -= 1;
						// Si ya no nos quedan vidas
						if (vidas == 0) {
							
							// Activamos los sonidos, si hemos
							// habilitado la msica por preferencias
							if (pref.getBoolean("musica", true)) { 
								// Reproducimos el sonido asociado 
								soundPool.play(idMuerte, 1, 1, 0, 0, 1); 
							}
							
							// Salimos del juego
							salir();
						}
					}
				}
			}
		}
		// Si no ha habido ninguna colisin
		if (!colision) {
			// Desmarcamos la colisin detectada anteriormente
			colisionProtaObstaculo = false;
		}
	}
		
	public void posicionaEscudo(int indiceEscudo) {
		// Contendr las coordenadas en las que colocar el escudo
		int centroXEscudo = 0;
		int centroYEscudo = -altoPantalla/2;
		Grafico escudo;
		escudo = escudos.elementAt(indiceEscudo);
		// Lo cambiaremos a true cuando vayamos a situar un escudo
		boolean escudoASituar = false;
		// Solo voy a situar dos escudos en pantalla
		if (indiceEscudo <=1 ) {		
			// Recorremos una zona del juego, tomando
			// como referencia las plataformas
			for (int i = ((int) ((indiceEscudo+1)*plataformas.size()/4)); 
					 i < ((int) ((indiceEscudo+2)*plataformas.size()/4)); 
					 i++) {
				// Si la plataforma est situada en la parte baja  
				if (plataformas.elementAt(i).getCenY() >= altoPantalla * 0.55) {
					// Situaremos el escudo por encima de esa plataforma
					centroYEscudo = (int) (altoPantalla * 0.25);
					centroXEscudo = plataformas.elementAt(i).getCenX();
					escudoASituar = true;
					break;
				// Si la plataforma est situada en la parte alta 
				} else if (plataformas.elementAt(i).getCenY() <= altoPantalla * 0.45) {
					// Situaremos el escudo debajo de esa plataforma
					centroYEscudo = (int) (altoPantalla * 0.75);
					centroXEscudo = plataformas.elementAt(i).getCenX();
					escudoASituar = true;
					break;
				}
			}
			// Situamos el escudo
			if (escudoASituar) {
				escudo.setCenX(centroXEscudo);
				escudo.setCenY(centroYEscudo);
			}
		// Si hay ms escudos los voy a situar fuera de la pantalla
		} else {
			escudo.setCenX(0);
			escudo.setCenY(-altoPantalla/2);			
		}
	}
		
	// Verificamos si hay colisin de prota y algn escudo
	public void verificaColisionProtaEscudos() {
		if (escudos.size() > 0) {
			// Recorremos escudos en orden inverso
			for (int i = escudos.size() - 1; i >= 0; i--) {
				Grafico escudo = escudos.elementAt(i);
				// Si el escudo est en la pantalla
				if (escudo.visible(anchoPantalla)) {
					// Si ha colisionado con el prota
					if (prota.verificaColision(escudo)) {						
						// Activamos los sonidos, si hemos
						// habilitado la msica por preferencias
						if (pref.getBoolean("musica", true)) {
							// Reproducimos el sonido asociado
							soundPool.play(idEscudo, 1, 1, 0, 0, 1);
						}  
								
						// Eliminamos el escudo
						escudos.remove(i);

						// Aadimos una vida
						vidas += 1;
					}
				}
			}
		}
	}
		
	public void activarSensores(){
		List<Sensor> listSensors;
				
		// Si vamos a usar los sensores de orientacin y proximidad 
		if (manejo == 2) {
			listSensors = mSensorManager.getSensorList(Sensor.TYPE_ORIENTATION);
			if (!listSensors.isEmpty()) {
				sensor = listSensors.get(0);
				mSensorManager.registerListener(this, sensor,
						SensorManager.SENSOR_DELAY_GAME);
			}
			
			listSensors = mSensorManager.getSensorList(Sensor.TYPE_PROXIMITY);
			if (!listSensors.isEmpty()) {
				sensor = listSensors.get(0);
				mSensorManager.registerListener(this, sensor,
						SensorManager.SENSOR_DELAY_GAME);
			}
		}

		// Si vamos a usar los sensores de aceleracin y proximidad
		if (manejo == 3) {
			listSensors = mSensorManager.getSensorList(Sensor.TYPE_ACCELEROMETER);
			if (!listSensors.isEmpty()) {
				sensor = listSensors.get(0);
				mSensorManager.registerListener(this, sensor,
						SensorManager.SENSOR_DELAY_GAME);
			}
			
			listSensors = mSensorManager.getSensorList(Sensor.TYPE_PROXIMITY);
			if (!listSensors.isEmpty()) {
				sensor = listSensors.get(0);
				mSensorManager.registerListener(this, sensor,
						SensorManager.SENSOR_DELAY_GAME);
			}
		}													
	}

	public void desactivarSensores(){
		mSensorManager.unregisterListener(this);		
	}
	
	@Override 
	public void onAccuracyChanged(Sensor sensor, int accuracy) {}
	 	 
	@Override 
	public void onSensorChanged(SensorEvent event) {
		switch(event.sensor.getType()) {
		case Sensor.TYPE_ORIENTATION:
			float valorY = event.values[1]; // Usamos sensor de eje Y
			if (!hayValorInicialY) {
				valorInicialY = valorY;
				hayValorInicialY = true;
			}
			// Si hemos movido significativamente
			// el terminal segn el eje Y
			if ((valorInicialY - valorY) > 5) {
				movimiento = AVANCE;
			} else if ((valorInicialY - valorY) < (-5)) {
				movimiento = RETROCESO;
			} else {
				movimiento = QUIETO;
			}
			break;
				
		case Sensor.TYPE_ACCELEROMETER:
			float valorMA = event.values[1];
			// Si el terminal deja de estar horizontal
			if (Math.abs(event.values[2] - GRAVEDAD) > 0.25) {
				// Si hemos movido significativamente
				// el terminal segn el eje Y
				if (valorMA > 0.5) {
					movimiento = AVANCE;
				} else if (valorMA < -0.5) {
					movimiento = RETROCESO;
				}
			} else {
				movimiento = QUIETO;
			}
			break;
				
		case Sensor.TYPE_PROXIMITY:
			// Se utiliza el criterio de que si el valor
			// del evento es 0, entonces nos hemos acercado
			// al sensor de proximidad hasta tocarlo.
			// Si el evento tiene otro valor, es que estamos
			// lejos del sensor

			// Si el detector de proximidad se ha activado
			if (event.values[0] == 0) {
				// Indicamos que el detector de proximidad se ha activado
				flagProx = true;
				// Si el detector de proximidad no detecta nada
			} else {
				// Si el flag de proximidad est activo
				// es que previamente nos hemos acercado
				if (flagProx) {
					// El prota salta
					protaSalta();
					// Indicamos que el sensor de proximidad ya no est activo
					flagProx = false;
				}
			}
			break;
		} 		
	}
	
	@Override
	public boolean onTouchEvent(MotionEvent event) {
		super.onTouchEvent(event);
		
		// Indicamos que, en principio,
		// no procesamos el evento
		boolean touchProcesado = false;		
		
		// Procesamos la pantalla tctil, si  
		// hemos habilitado la preferencia 
		if (manejo == 0) {	
			// Indicamos que procesamos el evento 
			touchProcesado = true;
			
			float x = event.getX();
			float y = event.getY();
			switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN:
			case MotionEvent.ACTION_MOVE:
				Rect rAvance    = avance.getDrawable().getBounds();
				Rect rRetroceso = retroceso.getDrawable().getBounds();
				Rect rSalto     = salto.getDrawable().getBounds();
				if (rAvance.contains((int) x, (int) y)) {
					movimiento = AVANCE;
				} else if (rRetroceso.contains((int) x, (int) y)) {
					movimiento = RETROCESO;
				} else if (rSalto.contains((int) x, (int) y)) {
					movimiento = QUIETO;
					protaSalta();					
				} else {
					movimiento = QUIETO;
				}
				break;
			case MotionEvent.ACTION_UP:					
				movimiento = QUIETO;
				break;								
			}
		}
		
		//return true;
		return touchProcesado;
	}
	
	
	@Override
	public boolean onKeyDown(int codigoTecla, KeyEvent evento) {
		super.onKeyDown(codigoTecla, evento);
		
		// Suponemos que vamos a procesar la pulsacin
		boolean procesada = true;
		
		// Procesamos los eventos de teclado
		// si hemos habilitado la preferencia
		if (manejo == 1) {
			switch (codigoTecla) {
			case KeyEvent.KEYCODE_DPAD_UP:
				// El prota salta
				protaSalta();
				break;
			case KeyEvent.KEYCODE_DPAD_LEFT:
				movimiento = RETROCESO;
				break;
			case KeyEvent.KEYCODE_DPAD_RIGHT:
				movimiento = AVANCE;
				break;
			default:
				// Si estamos aqu, no hay pulsacin que nos interese
				procesada = false;
				break;
			}
		} else {
			// Indicamos que no procesamos el evento
			procesada = false;
		}

		return procesada;
	}
	
	@Override
	public boolean onKeyUp(int codigoTecla, KeyEvent evento) {
		super.onKeyUp(codigoTecla, evento);
		
		// Suponemos que vamos a procesar la pulsacin
		boolean procesada = true;		
		
		// Procesamos los eventos de teclado  
		// si hemos habilitado la preferencia 
		if (manejo == 1) {
			switch (codigoTecla) {
			case KeyEvent.KEYCODE_DPAD_LEFT:
			case KeyEvent.KEYCODE_DPAD_RIGHT:
				movimiento = QUIETO;
				break;
			default:
				// Si estamos aqu, no hay pulsacin que nos interese
				procesada = false;
				break;
			}
		} else {
			// Indicamos que no procesamos el evento
			procesada = false;
		}
		return procesada;
	}
		
	public void setPadre(Activity padre) {
		this.padre = padre;
	}
	
	private void salir() {
		Bundle bundle = new Bundle();
		bundle.putInt("puntuacion", puntuacion);
		Intent intent = new Intent();
		intent.putExtras(bundle);
		padre.setResult(Activity.RESULT_OK, intent);
		padre.finish();
	}

	class ThreadJuego extends Thread {
		private boolean pausa, corriendo;

		public synchronized void pausar() {
			pausa = true;
		}

		public synchronized void reanudar() {
			pausa = false;
			notify();
		}

		public void detener() {
			corriendo = false;
			if (pausa)
				reanudar();
		}

		@Override
		public void run() {
			corriendo = true;
			while (corriendo) {
				actualizaFisica();
				synchronized (this) {
					while (pausa) {
						try {
							wait();
						} catch (Exception e) {
						}
					}
				}
			}
		}
	}	
	
	public ThreadJuego getThread() {
		return thread;
	}
}
